# mandayejs - saml reverse proxy
# Copyright (C) 2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import os
import subprocess
import urlparse

from django.conf import settings
from django.contrib.auth import views as auth_views
from django.contrib.auth import logout as auth_logout
from django.contrib.auth.decorators import login_required
from django.forms import PasswordInput
from django.forms import models as model_forms
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render, resolve_url
from django.template import RequestContext
from django.views.generic.base import TemplateView

from .models import Site, UserCredentials


def login(request, *args, **kwargs):
    return auth_views.login(request, *args, **kwargs)

def logout(request, *args, **kwargs):
    auth_logout(request)
    return HttpResponseRedirect('/')


class Panel(TemplateView):
    template_name = 'mandaye/panel.html'

panel = Panel.as_view()


@login_required
def post_login(request, *args, **kwargs):
    site = get_object_or_404(Site, domain=request.get_host())
    try:
        credentials = UserCredentials.objects.get(
                site=site,
                user=request.user)
    except UserCredentials.DoesNotExist:
        return HttpResponseRedirect(resolve_url('associate'))

    context = {}
    context['address'] = request.build_absolute_uri(site.login_path)
    return render(request, 'mandaye/post-login.html', context)


@login_required
def associate(request, *args, **kwargs):
    form_class = model_forms.modelform_factory(UserCredentials,
            fields=('username', 'password'),
            widgets={'password': PasswordInput()})
    if request.POST:
        credentials = UserCredentials()
        credentials.site = get_object_or_404(Site, domain=request.get_host())
        credentials.user = request.user
        form = form_class(request.POST,  instance=credentials)
    else:
        form = form_class()
    if not form.is_valid():
        return render(request, 'mandaye/associate.html', {'form': form})
    credentials.save()
    return HttpResponseRedirect(resolve_url('post-login'))


def post_login_do(request, *args, **kwargs):
    site = get_object_or_404(Site, domain=request.get_host())
    credentials = get_object_or_404(UserCredentials, site=site, user=request.user)
    login_info = {
            'username_locator': site.username_locator,
            'username_value': credentials.username,
            'password_locator': site.password_locator,
            'password_value': credentials.password,
            'address': request.build_absolute_uri(site.login_path),
            'cookies': [],
    }
    phantom = subprocess.Popen(['/usr/bin/phantomjs',
        '--ignore-ssl-errors=yes',
        '--ssl-protocol=any',
        os.path.join(settings.BASE_DIR, 'do_login.js')],
        close_fds=True,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE)
    stdout, stderr = phantom.communicate(json.dumps(login_info))
    result = json.loads(stdout)
    if result.get('result') != 'ok':
        return HttpResponseRedirect('/')
    response = HttpResponseRedirect('/')
    for cookie in result.get('cookies'):
        response.set_cookie(
                key=cookie.get('name'),
                value=cookie.get('value'),
                path=cookie.get('path'),
                secure=cookie.get('secure'),
                httponly=cookie.get('httponly'),
                )
    return response
